
import java.io.*;
import java.util.*;


/**
 *  File Utility Class
 */

public class FileUtil {

	
   /**
    * FileUtil() Constructor
    *
    * Don't let anyone instantiate this class
    */
	
	private FileUtil() {
	}
	

   /**
    * Returns an instance of a File
    *
    * @return        File
    */
	
	public static File getFile(String f) {
		
		return new File(f)	;		
	}


  /**
    * Returns an instance of a Property
    *
    * @parm          String				Property fileName
    * @return        Properties
    */
	
	public static Properties loadProperties(String f)	
					 throws PropertyFileNotFoundException {
		Properties prop = new Properties();
		
		try {
			FileInputStream fs = getFileInputStream(f);
			prop.load(fs);
			fs.close();
		}
		catch (FileNotFoundException pfnf) {
			throw new PropertyFileNotFoundException();			
		} 	
 		catch (IOException io) {
			throw new PropertyFileNotFoundException();			
		} 	
    	
		return prop;
	}	


   /**
    * Returns an instance of a FileInputStream
    *
    * @parm          File			 fileName
    * @return        FileInputStream
    */
	
	public static FileInputStream getFileInputStream(String f)
					  throws FileNotFoundException,  IOException {

		return new FileInputStream(f);
	}	


   /**
    * Returns an instance of a BufferedReader
    *
    * @parm          File			fileName
    * @return        BufferedReader
    */
	
	public static BufferedReader openInputFile(File f) 
				  throws FileNotFoundException,  IOException {

		return (new BufferedReader(new FileReader(f)));
	}	


   /**
    * Returns an instance of a BufferedWriter
    *
    * @parm          File			fileName
    * @return        BufferedWriter
    */

	public static BufferedWriter openOutputFile(File f) 
				  throws FileNotFoundException,  IOException {
		
		return (new BufferedWriter(new FileWriter(f)));
	}	



   /**
    * Closes the BufferedReader
    *
    * @parm          BufferedReader
    * @return        void
    */
	
	public static void close(BufferedReader br) {
	
		try {
			br.close();
		}
		catch(IOException io) {
		}
		
	}
	
	

   /**
    * Closes the BufferedWriter
    *
    * @parm          BufferedWriter
    * @return        void
    */
	
	public static void close(BufferedWriter bw) {
	
		try {
			bw.close();
		}
		catch(IOException io) {
		}
		
	}
	
   /**
    * println() Method
    *
    * Appends the object to the BufferedWriter
    *
    * @parm          BufferedWriter			
    * @parm          Object
    * @return        void 
    */
	
	public static void println(BufferedWriter bw, Object o) {

		try {
			bw.write(o + "\n");
		}
		catch(IOException io) {
		}
		
	}

}
