/*
 * $Id: CatalogDAO.java,v 1.17 2000/10/06 05:48:46 gmurray Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.shoppingcart.catalog.model;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import javax.naming.InitialContext;
import javax.sql.DataSource;
import javax.naming.NamingException;

import com.sun.j2ee.blueprints.shoppingcart.util.JNDINames;
import com.sun.j2ee.blueprints.shoppingcart.catalog.model.CatalogDAOException;
import com.sun.j2ee.blueprints.shoppingcart.catalog.model.ListChunk;
import com.sun.j2ee.blueprints.shoppingcart.util.Debug;
import com.sun.j2ee.blueprints.shoppingcart.util.DatabaseNames;

/**
 * This class encapsulates all the SQL calls made by Catalog EJB.
 * This layer maps the relational data stored in the database to
 * the objects needed by Catalog EJB.
*/
public class CatalogDAO {

    private  Connection     dbConnection;
    private  DataSource     datasource;

    public CatalogDAO()  throws CatalogDAOException {
        try {
            InitialContext ic = new InitialContext();
            datasource = (DataSource) ic.lookup(JNDINames.ESTORE_DATASOURCE);
        } catch (NamingException ne) {
            throw new CatalogDAOException("NamingException while looking " +
                            "up DB context  : " + ne.getMessage());
        }
    }


    public Category getCategory(String categoryId, Locale locale) throws
                                    CatalogDAOException {
        String qstr = "select catid, name, descn from "
          + DatabaseNames.getTableName(DatabaseNames.CATEGORY_TABLE, locale)
          + " where catid = '" + categoryId + "'";

        Category cat = null;
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            while (rs.next()) {
                int i = 1;
                String catid = rs.getString(i++).trim();
                String name = rs.getString(i++);
                String descn = rs.getString(i++);
                cat = new Category(catid, name, descn);
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                        "Category " + categoryId + " : " + se.getMessage());
        } finally {
            closeConnection();
        }
        return cat;
    }

    public ListChunk getCategories(int startIndex, int count,
                            Locale locale) throws CatalogDAOException {
        String qstr = "select catid, name, descn from " +
            DatabaseNames.getTableName(DatabaseNames.CATEGORY_TABLE, locale) +
            " order by name";

        int localCount = 0; // used to count the number of elems in list

        int num =
            getCount(DatabaseNames.getTableName(DatabaseNames.CATEGORY_TABLE,
                    locale));

        ArrayList al = new ArrayList();
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);

            // skip initial rows as specified by the startIndex parameter.
            while (startIndex-- > 0 && rs.next());

            // Now get data as requested.
            while (count-- > 0 && rs.next()) {
                int i = 1;
                String catid = rs.getString(i++).trim();
                String name = rs.getString(i++);
                String descn = rs.getString(i++);
                Category cat = new Category(catid, name, descn);
                al.add(cat);
                localCount++;
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                        "multiple categories : " + se.getMessage());
        } finally {
            closeConnection();
        }
        ListChunk rl = new ListChunk(num, al, startIndex, localCount);
        return rl;
    }

    public Product getProduct(String productId, Locale locale) throws
                                        CatalogDAOException {

        String qstr =
            "select productid, name, descn " +
            "from " +
            DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE, locale) +
            " where " +
            "productid='" + productId + "'";
        Debug.println("Query String is:" + qstr);

        Product product = null;
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            while (rs.next()) {
                int i = 1;
                String productid = rs.getString(i++).trim();
                String name = rs.getString(i++);
                String descn = rs.getString(i++);
                product = new Product(productid, name, descn);
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                        "product " + productId + " : " + se.getMessage());
        } finally {
            closeConnection();
        }
        return product;
    }

    public ListChunk getProducts(String categoryId, int startIndex,
                int count, Locale locale) throws CatalogDAOException {

        int localCount = 0 ;
        String qstr =
            "select productid, name, descn " +
            "from " +
            DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE, locale) +
            " where category =  " + "'" + categoryId + "' " + " order by name";
        Debug.println("Query String is:" + qstr);

        int num =
            getCount(DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE,
                locale) + " where category =  " + "'" + categoryId + "' ");

        ArrayList al = new ArrayList();
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            HashMap table = new HashMap();

            // skip initial rows as specified by the startIndex parameter.
            while (startIndex-- > 0 && rs.next());

            // Now get data as requested.
            while (count-- > 0 && rs.next()) {
                int i = 1;
                String productid = rs.getString(i++).trim();
                String name = rs.getString(i++).trim();
                String descn = rs.getString(i++).trim();

                Product product = null;
                if (table.get(productid) == null) {
                    product = new Product(productid, name, descn);
                    table.put(productid, product);
                    al.add(product);
                    localCount++;
                }
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                        "multiple products for cat " + categoryId + " : " + se.getMessage());
        } finally {
            closeConnection();
        }
        ListChunk rl = new ListChunk(num, al, startIndex, localCount);
        return rl;
    }

    public Item getItem(String itemId, Locale locale) throws
                                        CatalogDAOException {

        String qstr =
            "select itemid, listprice, unitcost, " +
            "attr1, a.productid, name, descn " +
            "from " +
            DatabaseNames.getTableName(DatabaseNames.ITEM_TABLE, locale) +
            " a, " +
            DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE, locale) +
            " b where " + "a.itemid = '" + itemId + "' order by name";
        Debug.println("Query String is:" + qstr);

        Item item = null;
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            while (rs.next()) {
                int i = 1;
                String itemid = rs.getString(i++).trim();
                double listprice = rs.getDouble(i++);
                double unitcost = rs.getDouble(i++);
                String attr1 = rs.getString(i++);
                String productid = rs.getString(i++).trim();
                String name = rs.getString(i++);
                String descn = rs.getString(i++);
                item = new Item(productid, itemid, attr1, listprice, unitcost);
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                        "Iten " + itemId + " : " + se.getMessage());
        } finally {
            closeConnection();
        }
        return item;
    }

    public ListChunk getItems(String productId, int startIndex, int count,
                                Locale locale) throws CatalogDAOException {

        int localCount = 0;
        String qstr =
            "select itemid, listprice, unitcost, " +
            "attr1, a.productid, name, descn " +
            "from " +
            DatabaseNames.getTableName(DatabaseNames.ITEM_TABLE, locale) +
            " a, " +
            DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE, locale) +
            " b where " + "a.productid = '" + productId +
            "' and a.productid = b.productid "+ " order by name";
        Debug.println("Query String is:" + qstr);

        int num =
            getCount(DatabaseNames.getTableName(DatabaseNames.ITEM_TABLE,
                locale) + " a, " +
                    DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE,
                        locale) + " b where a.productid = '" + productId +
                            "' and a.productid = b.productid");
        ArrayList al = new ArrayList();
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            HashMap table = new HashMap();

            // skip initial rows as specified by the startIndex parameter.
            while (startIndex-- > 0 && rs.next());

            // Now get data as requested.
            while (count-- > 0 && rs.next()) {
                int i = 1;
                String itemid = rs.getString(i++).trim();
                double listprice = rs.getDouble(i++);
                double unitcost = rs.getDouble(i++);
                String attr1 = rs.getString(i++);
                String productid = rs.getString(i++).trim();
                String name = rs.getString(i++);
                String descn = rs.getString(i++);
                Item item = new Item(productid, itemid, attr1, listprice,
                                                    unitcost);
                al.add(item);
                localCount++;
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                        "items for prod " + productId + " : " + se.getMessage());
        } finally {
            closeConnection();
        }
        ListChunk rl = new ListChunk(num, al, startIndex, localCount);
        return rl;
    }

    public ListChunk searchProducts(Collection keyWords, int startIndex,
                    int count, Locale locale) throws CatalogDAOException {
        int num = 0;
        int localCount = 0;
        String qstr1 =
            "select itemid, listprice, unitcost, " +
            "attr1, a.productid, name, descn " +
            "from " +
            DatabaseNames.getTableName(DatabaseNames.ITEM_TABLE, locale) +
            " a, " +
            DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE, locale) +
            " b where " + "a.productid = b.productid and " + "b.productid in ";
        String qstr = "";
        qstr += qstr1;
        qstr += getQueryStr ("PRODUCT", keyWords, locale);
        qstr += " UNION ";
        qstr += qstr1;
        qstr += getQueryStr( "CATEGORY", keyWords, locale);

        Debug.println("Query String is: " + qstr);

        ArrayList al = new ArrayList();
        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            HashMap table = new HashMap();
            // skip initial rows.
            while (startIndex-- > 0 && rs.next())
                num++;
            while (rs.next() && count > 0) {
                int i = 1;
                String itemid = rs.getString(i++).trim();
                double listprice = rs.getDouble(i++);
                double unitcost = rs.getDouble(i++);
                String attr1 = rs.getString(i++);
                String productid = rs.getString(i++).trim();
                String name = rs.getString(i++);
                String descn = rs.getString(i++);
                Product product = null;
                if (table.get(productid) == null) {
                    product = new Product(productid, name, descn);
                    table.put(productid, product);
                    al.add(product);
                    count--;
                }
                num++;
                localCount++;
            }
            rs.close();
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while searching : "+se.getMessage());
        } finally {
            closeConnection();
        }
        ListChunk rl = new ListChunk(num, al, startIndex, localCount);
        return rl;
    }

    private String getQueryStr(String table, Collection keyWords,
                                                Locale locale) {
        String qstr = new String();

        if (keyWords.size() < 1)
            return qstr;

        if (table.equals("PRODUCT")) {
            qstr = " ( SELECT productid from " +
                DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE, locale)+
                    " WHERE lower(name) like ";
            for (Iterator it = keyWords.iterator() ; it.hasNext() ; ) {
                qstr += "'%" + ((String)it.next()).toLowerCase() + "%'";
                if ((keyWords.size() > 1) && it.hasNext())
                    qstr += " OR lower(name) like ";
            }
            qstr += ")";

        }
        else if (table.equals("CATEGORY")) {
            qstr = " ( SELECT productid from " +
            DatabaseNames.getTableName(DatabaseNames.PRODUCT_TABLE, locale) +
            " c, " +
            DatabaseNames.getTableName(DatabaseNames.CATEGORY_TABLE, locale) +
            " d  WHERE c.category = d.catid and ";
            qstr += "  ( lower(d.name) like ";
            for (Iterator it = keyWords.iterator() ; it.hasNext() ; ) {
                qstr += "'%" + ((String)it.next()).toLowerCase() + "%'";
                if ((keyWords.size() > 1) && it.hasNext())
                    qstr += " OR lower(d.name) like ";
            }
            qstr += ") ) order by name ";
        }
        return qstr;
    }

    private int getCount(String tableEtAl) throws CatalogDAOException {
        String qstr = "select COUNT(*) from " +  tableEtAl;
        int count;

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet rs = stmt.executeQuery(qstr);
            rs.next();
            count = rs.getInt(1);
            stmt.close();
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                        "COUNT : " + se.getMessage());
        } finally {
            closeConnection();
        }
        return count;
    }

    private void getDBConnection() throws CatalogDAOException {
        try {
            dbConnection =  datasource.getConnection();
        } catch (SQLException se) {
            throw new CatalogDAOException("SQLException while getting " +
                            "DB connection  : " + se.getMessage());
        }
    }

    private void closeConnection() throws CatalogDAOException {
        try {
            if (dbConnection != null && !dbConnection.isClosed()) {
                dbConnection.close();
            }
        } catch(SQLException se) {
            throw new CatalogDAOException("SQLException while closing " +
                            "DB connection  : " + se.getMessage());
        }
    }
}
