/*
 * $Id: MainServlet.java,v 1.14 2000/10/10 10:37:23 gmurray Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.petstore.control.web;

import javax.servlet.ServletException;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.OutputStreamWriter;
import java.beans.Beans;
import java.io.IOException;
import java.util.HashMap;
import java.util.Locale;

import com.sun.j2ee.blueprints.util.tracer.Debug;
import com.sun.j2ee.blueprints.petstore.control.DuplicateAccountException;
import com.sun.j2ee.blueprints.petstore.control.web.ModelManager;
import com.sun.j2ee.blueprints.petstore.control.web.ScreenFlowXmlDAO;
import com.sun.j2ee.blueprints.petstore.control.web.RequestProcessor;
import com.sun.j2ee.blueprints.petstore.control.EStoreEventException;
import com.sun.j2ee.blueprints.petstore.util.WebKeys;


public class MainServlet extends HttpServlet {

    public void init() {
        Debug.println("MainServlet: Initializing");
        // we need to initialize the ScreenFlowManager first
        // because the RequestPrococessor depends on the ScreenFlowManager
        getScreenFlowManager();
        getRequestProcessor();
    }

    public  void doGet(HttpServletRequest request, HttpServletResponse  response)
        throws IOException, ServletException {
        String selectedURL = request.getPathInfo();
        // the current tomcat is resetting the outputstream so this is a workaround
        if (selectedURL.equals("/white")) return;
        HttpSession session = request.getSession();
        ScreenFlowManager screenManager = null;
        ModelManager modelManager= (ModelManager)request.getSession().getAttribute(WebKeys.ModelManagerKey);
        if ( modelManager == null ) {
            try {
                        modelManager = (ModelManager) Beans.instantiate(this.getClass().getClassLoader(), "com.sun.j2ee.blueprints.petstore.control.web.ModelManager");
             } catch (Exception exc) {
                 throw new ServletException ("Cannot create bean of class ModelManager");
             }
             session.setAttribute(WebKeys.ModelManagerKey, modelManager);
             modelManager.init(getServletContext(), session);
         }


         try {
             getRequestProcessor().processRequest(request);
             getScreenFlowManager().getNextScreen(request);
         } catch (DuplicateAccountException du){
             Debug.println("MainServlet: caught DupliateAccountException");
             session.setAttribute(WebKeys.CurrentScreen, "DUPLICATE_ACCOUNT");
         } catch (EStoreEventException ex){
             request.setAttribute("javax.servlet.jsp.jspException", ex);
             session.setAttribute(WebKeys.CurrentScreen, "ERROR");
         }
         Locale locale = (Locale)request.getSession().getAttribute(WebKeys.LanguageKey);
         if (locale == null) locale = Locale.US;
         /**
             Default to the base language or the site.
             If a language is found in the session use that template.
         */
         getServletConfig().getServletContext().getRequestDispatcher(getScreenFlowManager().getTemplate(locale)).forward(request, response);

    }

   private RequestProcessor getRequestProcessor() {
         RequestProcessor rp = (RequestProcessor)getServletContext().getAttribute(WebKeys.RequestProcessorKey);
         if ( rp == null ) {
             Debug.println("MainServlet: initializing reqeust processor");
             rp = new RequestProcessor();
             rp.init(getServletContext());
             getServletContext().setAttribute(WebKeys.RequestProcessorKey, rp);
        }
       return rp;
    }

    private ScreenFlowManager getScreenFlowManager() {
            ScreenFlowManager screenManager = (ScreenFlowManager)getServletContext().getAttribute(WebKeys.ScreenManagerKey);
            if (screenManager == null ) {
                Debug.println("MainServlet: Loading screen flow definitions");
                screenManager = new ScreenFlowManager();
                screenManager.init(getServletContext());
                getServletContext().setAttribute(WebKeys.ScreenManagerKey, screenManager);
             }
        return screenManager;
    }
}
