/*
 * $Id: RequestToEventTranslator.java,v 1.12 2000/09/21 06:51:35 brydon Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */
package com.sun.j2ee.blueprints.petstore.control.web;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.sun.j2ee.blueprints.petstore.control.EStoreEventException;
import com.sun.j2ee.blueprints.petstore.control.event.EStoreEvent;

import com.sun.j2ee.blueprints.util.tracer.Debug;
import com.sun.j2ee.blueprints.petstore.util.WebKeys;

import com.sun.j2ee.blueprints.petstore.control.web.handlers.RequestHandler;

/**
 * This class is responsible for processing all HTTP requests received from
 * the RequestProcessor and generating necessary events to modify
 * the model data.
 */
public class RequestToEventTranslator {

    private RequestProcessor sessionClientController;
    private ModelManager mm;
    private ScreenFlowManager screenManager;

    private static final String CATEGORY_ID = "categoryId";

    public RequestToEventTranslator(RequestProcessor sessionClientController,
                                                ScreenFlowManager screenManager) {
        this.sessionClientController = sessionClientController;
        this.screenManager = screenManager;
    }

    public EStoreEvent processRequest(HttpServletRequest request) throws EStoreEventException {
        // Process the request and get the necessary event depending on the URL
        String selectedUrl = request.getPathInfo();
        EStoreEvent event = null;
        String requestProcessorString = null;
        URLMapping urlMapping = screenManager.getURLMapping(request.getPathInfo());
        Debug.println("RequestToEventTranslator: url=" + selectedUrl);
        if (urlMapping != null) {
            requestProcessorString = urlMapping.getRequestHandler();
            if (urlMapping.useRequestHandler()) {
                RequestHandler handler = null;
                try {
                    Debug.println("RequestToEventTranslator: loading handler");
                    handler = (RequestHandler)getClass().getClassLoader().loadClass(requestProcessorString).newInstance();
                    handler.doStart(request);
                    event = handler.processRequest(request);
                    handler.doEnd(request);
               } catch (Exception ex) {
                   Debug.println("RequestToEventTranslator caught loading handler: " + ex);
               }
           }
        }
        return event;
    }
}
