package com.sun.j2ee.blueprints.petstore.util;

import com.sun.j2ee.blueprints.util.tracer.Debug;

import com.sun.j2ee.blueprints.petstore.control.DuplicateAccountException;
import com.sun.enterprise.security.auth.realm.*;
import com.sun.enterprise.security.auth.realm.sharedpassword.*;
import com.sun.enterprise.security.auth.Authenticator;
import com.sun.enterprise.security.AuthenticationServer;
import com.sun.enterprise.util.Utility;
import com.sun.enterprise.util.ORBManager;

import javax.servlet.http.HttpSession;
import com.sun.enterprise.security.LoginContext;
import com.sun.enterprise.security.LoginException;
import com.sun.enterprise.security.auth.AuthenticationStatus;

/**
 * This class implements the SecurityAdapter interface.
 * This class holds the security code specific to the
 * J2EE RI.
 * This class has the public field SECURITY_CONTEXT.
 *
 * @see com.sun.j2ee.blueprints.petstore.util.SecurityAdapter
 */
public class J2eeSecurityAdapter implements SecurityAdapter {

    public static final String SECURITY_CONTEXT = "SecurityContext";
    private static Authenticator auth = null;

    static {
        try {
            ORBManager.init(null, null);
            auth =(Authenticator)  Utility.lookupObject(
                                                        AuthenticationServer.AUTH_SERVICE_NAME,Authenticator.class);
        }catch(Exception e){
            Debug.println("J2eeSecurityAdapter Error Getting Authenticator: " +e);
            e.printStackTrace();
        }
    }


    /**
     * This method is used to add user to web tier realm. -
     * Currently this will only work in single vm mode
     * on the J2ee RI (this code is non portable).
     *
     * This code creates the specfied group in the realm if
     * it does not exist after which it will add the user to
     * the specified group.
     *
     * @throws  DupplicateAccountException if the userName
     *          exists.
     */
    public void addUser(String realmName, String group, String userName, String password) throws DuplicateAccountException{
        Realm r = null;
        try {
            r = Realm.getInstance(realmName);

        } catch (Throwable e) {
            e.printStackTrace();
        }

        if(r==null) {
            throw new RuntimeException("Unuable add user to web tier realm: realm non-existent");
        }

        try{
            Debug.println("Checking if user exists");

            java.util.Enumeration enum= r.getUserNames();
            while(enum.hasMoreElements()) {
                if (((String)enum.nextElement()).equals(userName)) throw new DuplicateAccountException("The user account" + userName + " already exists");
            }

            Debug.println("Checking groups");
            boolean groupExists = false;
            enum= r.getGroupNames();
            while(enum.hasMoreElements()) {
                if (((String)enum.nextElement()).equals(group)) groupExists = true;
            }
            // create the group if it does not exist
            if (!groupExists){
                ((SharedPasswordRealm)r).addGroup(group);
                Debug.println("Group " + group + " added successfully");
            }else Debug.println("Group " + group + " exitsts.");

            String [] groups = {group};
            ((SharedPasswordRealm)r).createUser(userName, password,  groups);
            Debug.println("Added User to " + realmName + " Realm userName=" + userName + " password=" + password + " group=" + group);

        }catch (BadRealmException be){
            be.printStackTrace();
        }
        try{
            if (auth != null){
                auth.refresh(r.getName());
                Debug.println("Refreshed " + r.getName() + " realm");
            } else Debug.println("Error refreshing " + r.getName() + " realm: Authenticator is null");
        }catch(Throwable e){
            Debug.println("Error authenicating user: " + e);
            e.printStackTrace();
        }
    }

    /**
     * This portion of code will log in a user into the
     * form based login mechanism of the sample application
     * when a new account is created.
     * This code is specific to the J2ee reference implementation.
     */
    public void loginUser(String userName, String password, HttpSession session) {
        Debug.println("Logging in user " + userName + " to web server");
        // log in the user
        try {
            LoginContext lc = new LoginContext();
            lc.login(userName, password);
        } catch(LoginException le) {
            le.printStackTrace();
            AuthenticationStatus as = le.getStatus();
        }

        if(session != null) {
            session.setAttribute(SECURITY_CONTEXT,
             com.sun.enterprise.security.SecurityContext.getCurrent());
        }
    }
}
